/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                Java Language Certus Mini A SDK, Version 7.3                */
/*                    Copyright 2024, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2024 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Automatic Magnetometer Calibration Status Packet */
public class ANPacket90
{
	public static final int LENGTH = 78;

	public static final int AUTOMATIC_MAGNETOMETER_CALIBRATION_DISABLED = 0;
	public static final int GNSS_AIDED_MAGNETOMETER_CALIBRATION = 1;
	public static final int ONLINE_MAGNETOMETER_CALIBRATION = 2;

	public int method;
	public boolean ready;
	public boolean coarseCompleted;
	public boolean existingCalibration;
	public boolean awaitingFilters;
	public boolean manualCalibrationInProgress;
	public boolean invalidVehicleType;
	public boolean magneticHeadingDisabled;

	public float convergence;
	public float[] scaleFactor;
	public float[] softIron;
	public float[] hardIron;
	public float[] scaleFactorStandardDeviation;
	public float[] softIronStandardDeviation;
	public float[] hardIronStandardDeviation;

	public ANPacket90(ANPacket packet)
	{
		scaleFactor	= new float[3];
		softIron = new float[3];
		hardIron = new float[3];
		scaleFactorStandardDeviation = new float[3];
		softIronStandardDeviation = new float[3];
		hardIronStandardDeviation = new float[3];

		method = packet.data[0] & 0xFF;
		ready = (packet.data[1] & 0x01) > 0;
		coarseCompleted = (packet.data[1] & 0x02) > 0;
		existingCalibration = (packet.data[1] & 0x04) > 0;
		awaitingFilters = (packet.data[1] & 0x08) > 0;
		manualCalibrationInProgress = (packet.data[1] & 0x10) > 0;
		invalidVehicleType = (packet.data[1] & 0x20) > 0;
		magneticHeadingDisabled = (packet.data[1] & 0x40) > 0;

		convergence = TypeConversion.bytesToFloat(packet.data, 2);

		for (int i = 0; i < 3; i++)
		{
			scaleFactor[i] = TypeConversion.bytesToFloat(packet.data, 6 + 4*i);
			softIron[i] = TypeConversion.bytesToFloat(packet.data, 18 + 4*i);
			hardIron[i] = TypeConversion.bytesToFloat(packet.data, 30 + 4*i);
			scaleFactorStandardDeviation[i] = TypeConversion.bytesToFloat(packet.data, 42 + 4*i);
			softIronStandardDeviation[i] = TypeConversion.bytesToFloat(packet.data, 54 + 4*i);
			hardIronStandardDeviation[i] = TypeConversion.bytesToFloat(packet.data, 66 + 4*i);
		}
	}
}
